package eye.ss.database;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.SequenceGenerator;

import eye.util.ObjectUtil;
import eye.util.logging.Log;

@SequenceGenerator(name = "seq", initialValue = 1, allocationSize = 100)
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
@Entity
@Cacheable
public abstract class JpaModel implements Comparable<JpaModel>, BaseModel {
	private String label;

	@Column(columnDefinition = "VARCHAR(2000)")
	private String description;

	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "seq")
	@Id
	public long id;

	public Integer rank = Integer.valueOf(0);

	@Override
	public int compareTo(JpaModel model) {
		int result = compareBody(model);
		assert result == -1 * model.compareBody(this) : this + " and " + model + " do not compare correctly";
		return result;
	}

	@Override
	public String getDescription() {
		return description;
	}

	@Override
	public long getId() {
		return id;
	}

	@Override
	public final String getLabel() {
		return label;
	}

	@Override
	public String getName() {
		return getClass().getSimpleName() + "_" + getId();
	}

	@Override
	public void setDescription(String description) {
		this.description = description;
	}

	@Override
	public void setLabel(String label) {
		this.label = label;
	}

	@Override
	public String toHandle() {
		if (label != null) {
			return getLabel();
		} else {
			return getName();
		}

	}

	@Override
	public String toString() {
		return getLabel();
	}

	@Override
	public void validate() {
	}

	private int compareBody(JpaModel model) {
		assert model != null : "Why is " + this + " being compared with a null object?";
		assert rank != null;
		assert model.rank != null;
		if (ObjectUtil.notEquals(rank, model.rank)) {
			// Higher ranges
			return Integer.compare(model.rank, rank);
		}

		String label = getLabel();
		String otherLabel = model.getLabel();
		if (label == null) {
			Log.warning("why does " + this + " have no label?");
			return 0;
		}
		if (otherLabel == null) {
			Log.warning("Why does " + model + " have no label?");
			return 0;
		}

		boolean labelInCat = label.contains(":");
		boolean otherInCat = otherLabel.contains(":");

		// If either both or neither in categories, we use the label.
		if (labelInCat == otherInCat) {
			return label.compareTo(otherLabel);
		} else if (labelInCat) {
			// When we have a label and it doesn't, we go first.
			return -1;
		} else {
			// When they have a label we go after them.
			return 1;
		}
	}

}
