package eye.ss.database;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import eye.service.EyeService;
import eye.service.ServiceEnv;
import eye.ss.ServerEnv;
import eye.transfer.EyeRecord;
import eye.transfer.EyeTable;
import eye.transfer.EyeType;
import eye.util.ExceptionUtil;
import eye.util.FileUtil;
import eye.util.FileVisitor;
import eye.util.Modelable;
import eye.util.StringUtil;
import eye.util.UserException;
import eye.util.charactoristic.HasMeta;
import eye.util.logging.Log;
import eye.util.logging.Log.Cat;

public abstract class Models<GChild extends Modelable> extends EyeService {

	private Class<GChild> myClass;

	public Models() {
		globalService = true;
	}

	public static void addSummaryRow(EyeTable table, String label, String description, String ownerLabel) {
		Object[] row = new Object[5];
		row[0] = label;
		row[1] = description;
		row[2] = ownerLabel;
		row[3] = -1l;
		row[4] = null;
		table.addRow(row);
	}

	public static void addToTable(EyeTable table, Modelable strat, String ownerLabel, EyeType type,
			Object... extraFields) {
		int size = 5;
		if (extraFields != null) {
			size += extraFields.length;
		}
		Object[] row = new Object[size];
		row[0] = strat.getLabel();
		row[1] = strat.getDescription();
		row[2] = ownerLabel;
		row[3] = strat.getId();
		row[4] = type;

		if (extraFields != null) {
			for (int i = 0; i < extraFields.length; i++) {
				row[i + 5] = extraFields[i];
			}
		}
		table.addRow(row);

	}

	public static EyeTable createModelTable() {
		EyeTable table = new EyeTable();
		table.setName("listings");
		table.addColumnNames("label", "description", "owner", "id");
		table.addColumnTypes(EyeType.String, EyeType.String, EyeType.String, EyeType.Long);
		table.addColumn("type", EyeType.String);
		return table;
	}

	public GChild create(String name, String label, String description) {
		GChild child = doCreate(name);
		child.setDescription(description);
		child.setLabel(label);
		return child;
	}

	public Query<GChild> createQuery() {
		return new Query(getChildClass(), db().getEm());
	}

	public GChild delete(long id) {
		db().checkTransaction(true);
		GChild child = get(id);
		if (child == null) {
			throw new UserException("Sorry, " + getEyeType().label() + " " + id + " was already removed", true);
		}
		child.delete(false);
		return child;
	}

	public GChild ensure(EyeRecord r) {

		Long id = r.getId();
		GChild strat = null;
		if (id == null) {
			String name = r.getName();
			if (StringUtil.hasContent(name)) {
				strat = createQuery().findByName(name);
			}
			if (strat == null) {
				String label = r.getLabel();

				db();
				if (DatabaseService.isInTransaction()) {
					if (r.get("master-course") == null && findByLabel(label) != null) {
						throw new UserException("You have already used " + label + ", so please rename your strategy",
								true);
					}
				}
				strat = doCreate(name);
			}
		} else {
			strat = createQuery().findById(id);

			assert strat != null : "There is no strategy with the id of " + id;
		}
		return strat;

	}

	public GChild find(EyeRecord record) {
		return find(record.getId());
	}

	public final GChild find(Long id) {
		assert id != null : "Must supply an id";
		GChild strat = createQuery().findById(id);
		if (strat == null) {
			throw new IllegalStateException("Using " + getClass().getSimpleName() + " " + getEyeType().label()
					+ " with the id " + id + " cannot be found");
		}
		return strat;
	}

	public List<GChild> findAll() {
		return createQuery().findAll();
	}

	public final GChild findGlobal(Long id) {
		assert id != null : "Must supply an id";
		GChild strat = createQuery().findById(id);
		if (strat == null) {
			throw new IllegalStateException(getEyeType().label() + " with the id " + id + " cannot be found");
		}
		return strat;
	}

	public final GChild get(EyeRecord record) {
		return get(record.getId());
	}

	public final GChild get(long id) {
		Query<GChild> q = createQuery();
		GChild strat = q.findById(id);
		return strat;

	}

	public Class<GChild> getChildClass() {
		if (myClass == null) {
			String name = this.getClass().getName();
			name = name.substring(0, name.length() - 1);
			try {
				myClass = (Class) Class.forName(name);
			} catch (ClassNotFoundException e) {
				throw ExceptionUtil.wrap(e);
			}
		}
		return myClass;
	}

	public abstract EyeType getEyeType();

	public List<GChild> getList() {
		List<GChild> list = new ArrayList();
		for (Object stratObj : createQuery().findAll()) {
			GChild strat = (GChild) stratObj;
			// We only want strategies associated with tactics here.
			if (isAvailable(strat)) {
				list.add(strat);
			}
		}
		return list;
	}

	public abstract EyeRecord getModels();

	@Override
	public void init() {

	}

	public final EyeRecord load(EyeRecord strat) {
		return load(find(strat));
	}

	public EyeRecord load(GChild strat) {
		EyeRecord r = createRecord(strat);
		r.setModel(strat);
		r.updateModel = false;
		bind(strat, r);
		return r;
	}

	public final EyeRecord load(Long id) {
		GChild strat = find(id);
		if (strat == null) {
			return null;
		} else {
			return load(strat);
		}
	}

	public void loadBlessed(String strackDir) {
		// Stracks.
		File f = new File(strackDir);
		if (f.exists() == false) {
			Log.warning(f + " does not exist, so exiting loading blessed");
			return;
		}

		assert f.isDirectory() : f.getAbsolutePath() + " should be a directory";
		FileVisitor visit = new FileVisitor() {

			@Override
			public boolean accept(File pathname) {
				return pathname.getName().endsWith(".txt");
			}

			@Override
			protected void processFile(File f) throws Exception {
				Log.config("start loading" + f, Cat.DATA);
				try {
					EyeRecord r = ServerEnv.loadRecord(FileUtil.load(f));
					String fileName = f.getName().substring(0, f.getName().indexOf("."));
					if (!r.getName().equals(fileName)) {
						throw new IllegalStateException("Error while loading " + fileName
								+ ". Record name should be the same not: " + r.getName());
					}
					// Void the id, if there is one, as it is not valid once the
					// db is reset
					r.clearIds();
					// The record needs a name.
					assert r.getName() != null : f.getName() + " should have included a name";
					save(r);
					Log.config("finished loading" + f, Cat.DATA);
				} catch (Exception e) {
					throw new IllegalStateException("Error while loading " + f, e);
				}
			}
		};
		FileUtil.process(visit, f);
	}

	public GChild save(EyeRecord r) {
		GChild strat = ensure(r);
		assert strat != null;
		r.updateModel = true;
		strat = bind(strat, r);
		assert strat != null : "How did record " + r.getName() + " fail to save?";
		strat.save();
		return strat;
	}

	protected abstract GChild bind(GChild strat, EyeRecord r);

	/** Adding flexible metadata storage, currently only used for pick filters. */
	protected void bindMeta(HasMeta strat, EyeRecord r) {
		if (r.updateModel) {
			for (Entry<String, Object> entry : r.contents.entrySet()) {
				if (entry.getKey().startsWith("meta-")) {
					if (strat.getMetadata() == null) {
						strat.setMetadata(new HashMap());
					}
					strat.getMetadata().put(entry.getKey(), entry.getValue());

				}
			}
		} else {
			if (strat.getMetadata() != null) {
				for (Entry<String, Object> entry : strat.getMetadata().entrySet()) {
					r.set(entry.getKey(), entry.getValue());

				}

			}
		}
	}

	protected abstract GChild create();

	protected EyeRecord createRecord(Modelable model) {
		EyeRecord record = new EyeRecord();
		record.setModel(model);
		return record;
	}

	protected final DatabaseService db() {
		return ServiceEnv.get().requireService(getDatabaseClass());
	}

	protected GChild findByLabel(String label) {
		return null;
	}

	protected List<GChild> findModels() {
		return createQuery().findAll();
	}

	protected abstract Class<? extends DatabaseService> getDatabaseClass();

	protected boolean isAvailable(GChild strat) {
		return strat.getClass().equals(getChildClass());
	}

	protected void setName(GChild child, String name) {
		throw ExceptionUtil.throwUnsupported();
	}

	private GChild doCreate(String name) {
		GChild child = create();
		if (StringUtil.hasContent(name)) {
			setName(child, name);
		}
		return child;
	}

}
