package models;

import static models.Contact.*;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.jdo.annotations.Index;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.Transient;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.HtmlEmail;

import eye.play.control.PlayControl;
import eye.play.control.PlayUtil;
import eye.service.ServiceEnv;
import eye.service.stock.Signal;
import eye.service.stock.TickerService;
import eye.service.stock.TickerService.Ticker;
import eye.ss.database.Models;
import eye.ss.database.Query;
import eye.transfer.EyeRecord;
import eye.transfer.EyeTable;
import eye.transfer.EyeType;
import eye.util.DateUtil;
import eye.util.ExceptionUtil;
import eye.util.FileUtil;
import eye.util.HtmlUtil;
import eye.util.ObjectUtil;
import eye.util.StringUtil;
import eye.util.logging.Log;
import eye.util.logging.Log.Cat;

@Entity
public class Account extends EyeModel {
	private static String emailBody;

	public static boolean IS_DOGFOOD = true;
	@Index
	public String name;
	public Date waitingForAuth;
	// DEFAULT benchmark
	public String benchmark = "SPY";
	public boolean publicAccess;
	public boolean admin;
	public String password;
	public Strategy tearSheet;
	/** Shadow logins are used by ShadowLogin to support aliases for accounts */
	public List<String> shadowLogins;

	public boolean whitelisted;
	@ManyToMany
	public List<Course> enrolled;
	@Index
	public String email;

	public Boolean paid;

	//Membership group.
	@Index
	public String mouseGroup;
	public String studentId;
	public String org;
	public List<String> adminForMouseGroups;
	public Date lastLogin;
	public Date emailUpdated;
	public Date lastUpdate;
	public List<Date> logins;

	public String brokerage;
	public String positions;
	public boolean acceptedTerms;
	/** Currently only used by trade king. Used to login with brokerage account. */
	@Index
	public String brokerageAccount;

	@Transient
	public transient Object[] homeRows;

	public int status;

	/**
	 * Permissions
	 */
	public boolean sendEmail;

	public boolean isMoneyManager;

	public List<Date> backtestDates;

	public Account(Contact contact, boolean publicAccount) {
		updateFromContact(contact);
		publicAccess = publicAccount;
		create();

	}

	/** Used for fake account */
	public Account(String userName) {
		setName(userName);
		label = "Fake:" + userName;

	}

	public Account(String userName, String label, String description, boolean globalAccount, String password) {
		this(new Contact(userName, password, label, description), globalAccount);
	}

	public boolean allowNewBacktest() {
		Date current = DateUtil.getPureDate();
		if (backtestDates == null) {
			backtestDates = new ArrayList();
		}

		if (backtestDates.size() > 0) {
			Date first = backtestDates.get(0);
			if (DateUtil.isWithinDay(first, current) == false) {
				backtestDates.clear();
			}
		}

		if (backtestDates.size() < 5) {
			backtestDates.add(current);
			return true;
		} else {
			return false;
		}

	}

	public void create() {
		save();
		PlayControl.getEnv().account = this;

	}

	public Query<Strategy> createGlobalQuery() {
		Query q = new Query(Strategy.class, DB.getEm());
		q.allowSubtypes = true;
		return q;
	}

	@Override
	public void delete(boolean isAccountDeleting) {
		Log.info("Delete account: " + this);
		deleteContents();
		super.delete(isAccountDeleting);
	}

	public void deleteContents() {
		List<Strategy> children = createGlobalQuery().findByOwner("account", this);
		for (Strategy child : children) {
			child.delete(true);
		}
		if (enrolled != null) {
			for (Course course : enrolled) {
				course.members.remove(this);
			}
		}
	}

	public final String emailAlerts() {
		return emailAlerts(false, null);
	}

	public String emailAlerts(boolean isTest, Date lastUpdate) {

		assert ServiceEnv.get() != null;
		if (lastUpdate != null) {
			emailUpdated = lastUpdate;
		}
		assert ServiceEnv.get() != null;

		emailBody = FileUtil.load("conf/emailTemplate.txt");
		TickerService service = PlayControl.getEnv().requireService(TickerService.class);
		service.ready();
		if (service.tickers == null) {
			Log.warning("Ticker service did not set up properly");
			service = PlayControl.getEnv().restartService(TickerService.class);
			assert service.tickers != null;
		}
		try {
			HtmlEmail email = new HtmlEmail();
			email.setFrom("support@equitieslab.com");
			if (getSignals().size() == 0) {
				return "no alerts";
			}

			List<StockPick> newBuys = new ArrayList();
			List<StockPick> oldBuys = new ArrayList();

			List<StockPick> newSells = new ArrayList();
			List<StockPick> oldSells = new ArrayList();

			for (StockPick pick : getSignals()) {
				if (pick.isPending()) {
					if (emailUpdated == null || pick.signalDate.after(emailUpdated)) {
						if (pick.isBuy()) {
							newBuys.add(pick);
						} else {
							newSells.add(pick);
						}
					} else {
						if (pick.isBuy()) {
							oldBuys.add(pick);
						} else {
							oldSells.add(pick);
						}
					}
				}
			}
			StringBuilder accum = new StringBuilder();
			if (newBuys.size() + newSells.size() == 0) {

				Log.config(this + "has no alerts, email updated " + ObjectUtil.format(emailUpdated), Cat.MAIL);

			}
			String todayDate = ObjectUtil.format(new Date());

			String prefix = "As of " + todayDate + ": ";
			emitRows(prefix + "New Buy", service, newBuys, accum);
			emitRows(prefix + "New Sell", service, newSells, accum);

			emitRows("Older Buy", service, oldBuys, accum);
			emitRows("Older Sell", service, oldSells, accum);

			email.setSubject("Your alerts for " + todayDate);
			String msg = String.format(emailBody, label, accum);
			email.setMsg(msg);

			if (PlayUtil.isProd() == false && isInternal() == false && isTest == false) {
				String mesg = "Skipping sending real e-mail to " + name;
				Log.config(mesg, Cat.MAIL);
				return msg;
			}
			if (IS_DOGFOOD) {
				if (isInternal()) {

					if (this.email == null) {
						email.addTo("hemilycrutcher@yahoo.com");
					} else {
						email.addTo(this.email);
					}
				} else {
					Log.fine("disabled mail non system:" + email.getToAddresses(), Cat.MAIL);
					return "Disabled non system accounts";
				}
			} else {
				if (this.email == null) {
					Log.config("no email set up for  " + this, Cat.MAIL);
					return null;
				} else {
					email.addTo(this.email);
				}
			}
			if (isTest == false) {
				Log.config("sending to " + email.getToAddresses() + " for account " + label, Cat.MAIL);
				//COVERT_TODO: Mail.send(email);
			} else {
				Log.config("supressed sending to " + email.getToAddresses(), Cat.MAIL);
			}
			emailUpdated = DateUtil.getPureDate();

			return msg;
		} catch (EmailException e) {
			Log.warning(e);
			throw ExceptionUtil.wrap(e);
		}

	}

	@Override
	public boolean equals(Object e) {
		if (this == e) {
			return true;
		}
		if (e instanceof Account) {
			Account other = (Account) e;
			if (name != null && other.name != null) {
				return name.equals(other.name);
			}
		}
		return false;
	}

	// Needs to be expanded.
	public void exportBlessed(File root) {
		assert root.isDirectory();
		root = FileUtil.ensureEmptyDir(root, getName());
		exportBlessed(root, PlayControl.pickFilters);
		exportBlessed(root, PlayControl.tradingModels);
		exportBlessed(root, PlayControl.macros);
		exportBlessed(root, PlayControl.watchlists);

	}

	public EyeTable getHoldings() {
		List<StockPick> picks = getStockPicks();

		EyeTable table = new EyeTable();
		table.setName("holdings");
		table.addColumn("TickerId", EyeType.String);
		table.addColumn("Watchlist", EyeType.Long);
		table.addColumn("score", EyeType.Float);
		table.addColumn("score_from", EyeType.Date);
		table.addColumn("action", EyeType.String);
		table.addColumn("reason", EyeType.StringList);
		table.addColumn("Watchlist_Label", EyeType.String);
		table.addColumn("PickId", EyeType.Long);

		HashMap<Watchlist, Boolean> includes = new HashMap();
		for (StockPick pick : picks) {
			Watchlist list = pick.watchlist;
			Boolean include = includes.get(list);
			if (list != null) { // Henry HACK:  Why is list null
				if (include == null) {
					if (list.getLabel() != null) {
						String label = list.getLabel().toLowerCase();
						include = !StringUtil.containsAny(label, "experimental:", "failed:", "deprecated:");
						includes.put(list, include);
					}
				}
				if (include != null && include) {
					if (pick.isHeld() || pick.signal().isEnter()) {
						Date scored = Watchlist.getScoreFromDate(pick);
						table.addRow(pick.security, pick.watchlist.id, 0, scored, pick.signal, pick.signalCause,
								pick.getSimpleWatchlistLabel(), pick.id);
					}
				}
			}
		}

		if (positions != null) {
			int counter = 0;
			EyeRecord r = EyeRecord.load(positions);
			EyeTable positions = r.require("positions");
			Date scored = PlayUtil.watchlistScored;

			for (int row = 0; row < positions.getRowCount(); row++) {
				String tickerId = positions.get(row, 0);
				String watchlist = positions.getValueAt(row, "brokerage");
				Date bought = positions.getValueAt(row, "start");
				if (bought.before(scored)) {
					bought = scored;
				}
				table.addRow(tickerId, 0, 0, bought, Signal.hold, null, watchlist, ++counter);
			}
		}
		return table;
	}

	public Date getLastTouched() {
		if (lastLogin == null) {
			return lastUpdate;
		} else if (lastUpdate == null) {
			return lastLogin;
		} else if (lastUpdate.before(lastLogin)) {
			return lastLogin;
		} else {
			return lastUpdate;
		}
	}

	public <X extends EyeModel> List<X> getMembers(Models<X> service) {
		List<X> picks = service.createQuery().findByOwner("account", this);
		return picks;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public EyeModel getOwner() {
		return null;
	}

	public List<PickFilter> getPickFilters() {
		return getMembers(PlayControl.pickFilters);
	}

	// Gets all the stockpicks associated with the account.
	public List<StockPick> getSignals() {
		List<StockPick> alerts = new ArrayList();

		for (Watchlist list : getWatchlists()) {
			for (StockPick pick : list.ensurePicks().values()) {
				if (pick.isPending()) {
					alerts.add(pick);
				}

			}
		}

		return alerts;
	}

	public String getStatusLabel() {
		switch (status) {
		case STATUS_UNKNOWN:
			return "unknown";
		case STATUS_ACTIVE:
			return "active";
		case STATUS_CANCELED:
			return "canceled";
		case STATUS_PAUSED:
			return "paused";
		case STATUS_ERROR:
			return "error";
		case STATUS_PENDING_ACTIVATION:
			return "pending activation";

		case STATUS_PENDING_CANCELLATION:
			return "pending cancellation";

		case STATUS_EXPIRED:
			return "expired";
		case STATUS_OVERDUE:
			return "overdue";
		default:
			throw new IllegalStateException(status + " is unknown");
		}
	}

	public List<StockPick> getStockPicks() {
		return getMembers(PlayControl.stockPicks);
	}

	public String getStudentIdentifier() {
		if (StringUtil.hasContent(studentId)) {
			return studentId;
		} else {
			return "";
		}
	}

	public List<Strack> getTradingModels() {
		return getMembers(PlayControl.tradingModels);
	}

	public String getUpgradeArgs() {
		if (isAdmin()) {
			return "&first_name=henry&last_name=crutcher&email=henrycrutcher@gmail.com";
		} else {
			String[] names = label.split(" ");
			if (names.length != 2) {
				return "";
			}
			return "&first_name=" + names[0] + "&last_name=" + names[1] + "&email=" + email;
		}
	}

	public List<Watchlist> getWatchlists() {
		return getMembers(PlayControl.watchlists);

	}

	@Override
	public int hashCode() {
		return name.hashCode();
	}

	public boolean isAdmin() {
		return publicAccess || admin;

	}

	public boolean isBasicAccount() {
		return "TradeKing".equals(org);
	}

	public boolean isInternal() {

		return "EmilyCrutcher NickStephan sugata.ray@gmail.com HenryCrutcher Henry Global support@equitieslab.com"
				.contains(name);
	}

	public boolean isProfessor() {
		return mouseGroup != null && StringUtils.containsIgnoreCase(mouseGroup, "professor")
				|| studentId != null && studentId.equals("professor");
	}

	public boolean isStudent() {
		return mouseGroup != null && StringUtils.containsIgnoreCase(mouseGroup, "student")
				|| studentId != null && !(studentId.equals("pro") || studentId.equals("professor"));
	}

	public void replaceCat(String cur, String rep) {

		List<Strategy> children = createGlobalQuery().findByOwner("account", this);
		for (Strategy child : children) {

			String label = child.getLabel();
			int catPrefix = label.indexOf(":");
			if (catPrefix != -1) {
				String cat = label.substring(0, catPrefix);
				if (cat.equalsIgnoreCase(cur)) {
					String newLabel = rep + label.substring(catPrefix);
					child.label = newLabel;
				}
			}
		}
	}

	public void setLastLogin(Date d) {
		assert d != null;
		if (lastLogin != null) {
			if (Math.abs(d.getTime() - lastLogin.getTime()) < 86400000) {
				return;
			}
		}
		DB.startTransaction();
		lastLogin = d;
		if (logins == null) {
			logins = new ArrayList();
		}
		while (logins.size() > 5) {
			logins.remove(0);
		}
		logins.add(d);
		DB.commit();
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setUniqueName(String name) {
		if (name.equals(this.name) == false) {
			assert checkUniqueName(name);
			this.name = name;
		}
	}

	public boolean shouldEmail() {
		return sendEmail;
	}

	public String toPrettyString() {
		return StringUtil.pad(label, 30) + StringUtil.pad(ObjectUtil.format(getLastTouched()), 20);
	}

	@Override
	public String toString() {
		return name;
	}

	public void update(Contact contact, StringBuilder log) {
		boolean same = true;
		if (contact.email != null && !contact.email.equals(email)) {
			log.append(email + " needs to be updated from " + contact.email);
			same = false;
		} else if (contact.userName != null && contact.userName.equals(name) == false) {
			same = false;
			log.append(name + " needs to be updated from " + contact.userName);
		} else if (contact.label != null && !contact.label.equals(label)) {
			log.append(label + " needs to be updated from " + contact.label);
			same = false;
		} else if (contact.password != null && contact.password.equals(password) == false) {
			log.append(label + " needs to be updated from " + contact.password);
			same = false;
		} else if (contact.mouseGroup != null && contact.mouseGroup.equals(mouseGroup) == false) {
			log.append("Mouse group needs to be updated from " + contact.mouseGroup);
			same = false;
		} else if (contact.status != status) {
			log.append("Status updated from " + contact.status);
			same = false;
		} else if (StringUtil.equals(mouseGroup, contact.mouseGroup)) {
			same = false;
		}

		if (same == false) {
			log.append("\n Updating " + contact);
			DB.startTransaction();
			updateFromContact(contact);
			DB.commit();
		}

	}

	protected void emitRows(String title, TickerService service, List<StockPick> newPicks, StringBuilder accum) {
		if (newPicks.size() == 0) {
			return;
		}
		if (newPicks.size() > 1) {
			title = title + "s";
		}
		Collections.sort(newPicks, new Comparator<StockPick>() {

			@Override
			public int compare(StockPick me, StockPick other) {
				if (me.signalDate != null && !me.signalDate.equals(other.signalDate)) {
					return other.signalDate.compareTo(me.signalDate);
				} else if (!other.watchlist.equals(me.watchlist)) {
					return me.watchlist.compareTo(other.watchlist);
				} else {
					return me.security.compareTo(other.security);
				}
			}
		});

		HtmlUtil.toggle.set(false);
		accum.append("<br/><b>" + title + "</b><br/>");
		accum.append("<table>");
		for (StockPick pick : newPicks) {
			Ticker ticker = service.getTickerById(pick.security);
			if (ticker == null) {
				Log.warning(pick.security + " is not a known ticker");
			} else {
				HtmlUtil.emitRow(Signal.parse(pick.signal).getTitle() + " " + ticker.getTicker(), accum,
						ticker.getCompany(), pick.watchlist.getSimpleLabel(),
						DateUtil.displayFormat.format(pick.signalDate));
			}
		}
		accum.append("</table>");
	}

	private boolean checkUniqueName(String name) {
		if (createQuery().isNameUnique(name)) {
			return true;
		} else {
			throw new IllegalStateException(name + " must be unique. Current name is " + getName());
		}

	}

	private <X extends EyeModel> void exportBlessed(File root, Models<X> memberType) {
		String name = memberType.getType();
		File memberDir = FileUtil.ensureEmptyDir(root, name);
		for (X member : getMembers(memberType)) {
			String fileName = member.getLabel();
			if (fileName.contains(":")) {
				String cat = fileName.substring(0, fileName.indexOf(":"));
				cat = StringUtil.javaNormalForm(cat);
				fileName = cat + "/" + member.getName();
			} else {
				fileName = member.getName();
			}
			fileName = memberDir.getPath() + "/" + fileName;
			String content = memberType.load(member).toYaml();
			FileUtil.saveFile(fileName, content);
		}
	}

	private void updateFromContact(Contact contact) {
		setUniqueName(contact.userName);
		label = contact.label;
		if (contact.description != null) {
			description = contact.description;
		} else {
			description = "Account for " + label;
		}
		if (contact.password != null) {
			password = contact.password;
		}

		mouseGroup = contact.mouseGroup;
		if (mouseGroup != null) {
			if ("platinum".equals(mouseGroup) || StringUtils.containsIgnoreCase(mouseGroup, "institutional")) {
				isMoneyManager = true;
			}
		}

		if (contact.email != null) {
			email = contact.email;
		}
		if (contact.status != 0) {
			status = contact.status;
			if (!getStatusLabel().equalsIgnoreCase(contact.statusLabel)) {
				Log.warning(getStatusLabel() + " should be " + contact.statusLabel);
			}
		}
	}

	public static class BuySellSignal {
		public String name;

		public BuySellSignal(String name) {
			this.name = name;
		}
	}

}
