package models;

import java.util.Collections;
import java.util.List;

import eye.play.control.PlayControl;
import eye.ss.database.Query;
import eye.util.ExceptionUtil;
import eye.util.UserException;
import eye.util.logging.Log;

public abstract class AccountMembers<GChild extends EyeModel> extends PlayModels<GChild> {

	@Override
	public final GChild create() {
		Account account = PlayControl.getEnv().account;
		GChild child;
		try {
			child = getChildClass().newInstance();
		} catch (InstantiationException e) {
			throw ExceptionUtil.wrap(e);
		} catch (IllegalAccessException e) {
			throw ExceptionUtil.wrap(e);
		}
		setAccount(child, account);
		Log.info("Using create with save for " + this);
		return child;

	}

	@Override
	public GChild findByLabel(String label) {
		GChild cand = createQuery().findByLabel(label, "account", PlayControl.getEnv().account);
		if (cand == null) {
			cand = createQuery().findBySubstring(label, PlayControl.getEnv().account);
		}
		return cand;
	}

	@Override
	public List<GChild> findModels() {
		return createQuery().findByOwner("account", PlayControl.getEnv().account);
	}

	public GChild get(String name) {
		Query<GChild> q = createQuery();
		GChild strat = q.findByName(name, "account", PlayControl.getEnv().account);
		return strat;
	}

	public GChild getGlobal(String name) {

		Query<GChild> q = createQuery();
		GChild strat = q.findByName(name);
		return strat;
	}

	public List<GChild> getModels(Account target) {
		List<GChild> stratList = createQuery().findByOwnerWithQuery("account", target, "target.master IS NULL");
		return stratList;
	}

	public GChild require(String name) {
		GChild strat;
		strat = get(name);
		if (strat == null) {
			throw new UserException("The strategy with the name " + name + " cannot be found.", false);
		}

		return strat;
	}

	public void sort(final List<GChild> list) {
		if (list.size() > 0) {
			Collections.sort(list);
		}
	}

	protected abstract void setAccount(GChild child, Account account);

}
