package models;

import java.math.BigInteger;
import java.security.MessageDigest;

import eye.util.ExceptionUtil;

public class Contact {

	private static String CODE = "904e466e73606aabc523f630864d5f2a";
	public final static int STATUS_UNKNOWN = 0;
	public final static int STATUS_ACTIVE = 1;
	public final static int STATUS_CANCELED = 2;
	public final static int STATUS_PAUSED = 4;
	public final static int STATUS_ERROR = 7;
	public final static int STATUS_PENDING_ACTIVATION = 6;
	public final static int STATUS_PENDING_CANCELLATION = 9;
	public final static int STATUS_EXPIRED = 8;
	public final static int STATUS_OVERDUE = 5;

	public String label;

	public String userName;

	public String password;

	public String email;

	public String description;
	public String signUpUrl;

	public String org;

	public Account account;

	// Random content depending upon page.
	public String content;
	// Would use org, but org still is being used to specify non-membermouse.
	public String mouseGroup;
	public int status;
	public String statusLabel;

	public Contact() {
	}

	public Contact(String userName, String password, String label, String description) {
		this.userName = userName;
		this.password = password;
		this.label = label;
		this.description = description;
	}

	public static boolean shouldDelete(int status) {
		switch (status) {
		case STATUS_CANCELED:
		case STATUS_ERROR:
		case STATUS_EXPIRED:
			return true;
		default:
			return false;
		}
	}

	public void connectWith(Account account) {
		this.account = account;
		label = account.getLabel();

	}

	public String getCancelUrl() {
		return getUrl("cancel");
	}

	public String getSecretCode() {
		return CODE;
	}

	public String getUpdateUrl() {
		return getUrl("update");
	}

	public String getUrl(String action) {
		try {
			String hash = userName + "|" + getSecretCode();

			byte[] bytesOfMessage = hash.getBytes();
			MessageDigest md = MessageDigest.getInstance("MD5");
			byte[] thedigest = md.digest(bytesOfMessage);

			BigInteger bigInt = new BigInteger(1, thedigest);
			String result = bigInt.toString(16);
			// Now we need to zero pad it if you actually want the full 32 chars.
			while (result.length() < 32) {
				result = "0" + result;
			}
			result = result.substring(0, Math.min(result.length(), 10));
			String html = "https://equitieslab.chargevault.com/" + action + "?code=" + userName + "&key=" + result;
			return html;
		} catch (Exception e) {
			throw ExceptionUtil.wrap(e);
		}
	}

	public boolean shouldAccountBeCreated() {
		return status == STATUS_ACTIVE;
	}

	@Override
	public String toString() {
		String r = "username:" + userName;
		r += ", email:" + email;
		r += ", password:" + password;
		r += " label:" + label;
		return r;

	}

	public void trim() {
		label = label.trim();
		password = password.trim();
		userName = userName.trim();
		if (org != null) {
			org = org.trim();
		}

	}
}
