package models;

import static eye.EyeConstants.*;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.jdo.annotations.Index;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;

import eye.play.control.PlayControl;
import eye.ss.database.Models;
import eye.transfer.EyeTable;
import eye.transfer.EyeType;
import eye.util.DateUtil;
import eye.util.NumberUtil;
import eye.util.ObjectUtil;
import eye.util.collection.ListUtil;
import eye.util.logging.Log;

// Used only for courses for now.
@Entity
public class Course extends Strategy {

	private static Date BEFORE_START = DateUtil.parseUserDate("1-1-2000");

	@Index
	@ManyToMany
	public List<Account> members;

	@OneToMany
	public List<PickFilter> homeworks;

	public String name;

	public void addHomework(PickFilter strategy) {
		if (homeworks == null) {
			homeworks = new ArrayList();
		} else {
			cleanHomeworks();
		}
		if (homeworks.contains(strategy) == false) {
			homeworks.add(strategy);
		}
		homeworks.sort(new Comparator<PickFilter>() {

			@Override
			public int compare(PickFilter o1, PickFilter o2) {
				return o1.label.compareTo(o2.label);
			}
		});
		lastModified = new Date();
	}

	public void cleanHomeworks() {

		for (int i = 0; i < homeworks.size(); i++) {
			PickFilter homework = homeworks.get(i);
			if (homework == null || homework.label == null) {
				DB.ensureTransaction();
				homeworks.remove(i);
				--i;
			}
		}

	}

	@Override
	public int compareTo(EyeModel model) {
		Course other = (Course) model;
		int rankDif = ObjectUtil.compare(model.rank, rank);
		if (rankDif != 0) {
			return rankDif;
		}

		return ObjectUtil.compare(other.lastModified, lastModified);
	}

	public EyeTable createGradeTable() {

		EyeTable table = createStudentTable();
		table.setName("grade-table");
		if (ListUtil.hasContent(homeworks)) {
			cleanHomeworks();
			ListMultimap<PickFilter, String> extraColumns = ArrayListMultimap.create();
			for (PickFilter homework : homeworks) {

				table.addColumn(homework.getSimpleLabel() + ": grade", EyeType.Percent);
				table.addColumn(homework.getSimpleLabel() + ": submitted", EyeType.DateTime);
				table.addColumn(homework.getSimpleLabel() + ": ref", EyeType.homework);
				//Homework specific metadata.
				HashMap<String, Object> metadata = homework.getMetadata();

				for (Entry<String, Object> entry : metadata.entrySet()) {
					String key = entry.getKey();
					if (key.startsWith("meta-export-")) {
						Boolean value = (Boolean) entry.getValue();
						if (value == true) {
							String flavor = key.replace("meta-export-", "");
							String type = (String) metadata.get("meta-type-" + flavor);
							if (type == null) {
								throw new IllegalStateException(
										"Why does " + flavor + " have no type?" + "\n all keys:" + metadata.keySet());
							}
							EyeType colType = EyeType.parse(type);
							String label = (String) metadata.get("meta-label-" + flavor);
							extraColumns.put(homework, flavor);
							table.addColumn(homework.getSimpleLabel() + ":" + label, colType);
						}
					}
				}

			}
			table.adjustRowSizes();
			final HashMap<String, Integer> rowMap = new HashMap();
			for (int i = 0; i < table.getRowCount(); i++) {
				rowMap.put((String) table.getValueAt(i, 0), i);
			}

			for (PickFilter homework : homeworks) {
				List<String> extras = extraColumns.get(homework);

				List<PickFilter> assignments = PlayControl.pickFilters.getAllOwned(homework);
				for (PickFilter assignment : assignments) {

					Boolean hidden = assignment.getMeta("meta-hide-homework");
					if (hidden != null && hidden) {
						continue;
					}
					assert homework.equals(assignment.master) : assignment + " is bad for " + homework;
					int col = table.findColumn(homework.getSimpleLabel() + ": grade");
					assert assignment.account != null;
					String accountName = assignment.account.getName();
					assert accountName != null;
					assert rowMap != null;
					Integer row = rowMap.get(accountName);
					if (row == null) {
						Log.warning(assignment.account + " no longer a student in the course");
						DB.ensureTransaction();
						assignment.delete(true);
						continue;
					}
					assert row != null : accountName + " was not found";
					addToTable(table, assignment, "grade", row, col);
					addToTable(table, assignment, "submitted", row, ++col);
					table.setValueAt(assignment.getId(), row, ++col);
					for (String meta : extras) {
						addToTable(table, assignment, meta, row, ++col);
					}
				}
			}
		}
		return table;
	}

	public EyeTable createHomeworkTable() {

		EyeTable table = Models.createModelTable();
		table.getColumnNames().set(0, "label");
		if (isBrowsing()) {
			table.addColumn("homework status", EyeType.String);
		}
		if (homeworks != null) {
			for (int i = 0; i < homeworks.size(); i++) {
				PickFilter strat = homeworks.get(i);
				if (strat == null || strat.label == null) {
					DB.ensureTransaction();
					homeworks.remove(i);
					DB.commit();
					--i;
					continue;
				}

				if (isBrowsing()) {
					addStudentHomework(table, strat);
				} else {
					Models.addToTable(table, strat, strat.getLabel(), EyeType.homework);
				}
			}
		}
		table.setName("homeworks");
		return table;

	}

	public EyeTable createStudentTable() {
		EyeTable table = new EyeTable();
		table.setName("student-table");
		table.addColumn("email", EyeType.String);
		table.addColumn("student id", EyeType.String);
		table.addColumn("label", EyeType.String);
		table.addColumn("last login", EyeType.Date);
		table.addColumn("owner", EyeType.String);
		if (members != null) {
			for (Account account : members) {
				table.addRow(account.name, account.getStudentIdentifier(), account.label, account.lastLogin,
						account.getName());
			}
		}
		return table;
	}

	@Override
	public void delete(boolean isAccountDeleting) {
		super.delete(isAccountDeleting);

		if (!isAccountDeleting) {
			for (Account account : PlayControl.accounts.findAll()) {
				if (account.enrolled != null) {
					account.enrolled.remove(this);
				}
			}
		}
	}

	@Override
	public EyeModel getOwner() {
		return account;
	}

	public boolean isBrowsing() {
		return !PlayControl.getEnv().account.equals(account);
	}

	@Override
	public final String toHandle() {
		return name + " taught by " + account.getLabel();
	}

	@Override
	public String toString() {
		return label + " teacher: " + account + " size: " + (members == null ? "none" : members.size());
	}

	private void addStudentHomework(EyeTable table, PickFilter assignment) {
		Boolean hidden = assignment.getMeta("meta-hide-homework");
		if (hidden != null && hidden) {
			return;
		}
		List<PickFilter> homework = PlayControl.pickFilters.getMyOwned(assignment.id);
		Object value;

		if (ListUtil.isEmpty(homework)) {
			value = "not started";
		} else {
			PickFilter filter = homework.get(0);
			Double score = (Double) filter.getMetadata().get(META_HOMEWORK_GRADE);
			if (score != null && score > 0) {
				if (score == 100) {
					value = "<html><b><font color=green>perfect!</font></b></html>";
				} else {
					value = "submitted, " + NumberUtil.formatAsPercent(score);
				}
			} else {
				value = "in process";
			}
		}
		Models.addToTable(table, assignment, assignment.getLabel(), EyeType.homework, value);
	}

	private void addToTable(EyeTable table, PickFilter assignment, String suffix, int row, int col) {
		Object value = assignment.getMeta("meta-homework-" + suffix);
		if (value == null) {
			value = Double.NaN;
		}
		if (value instanceof Date) {
			if (BEFORE_START.after((Date) value)) {
				value = null;
			}

		}
		table.setValueAt(value, row, col);

	}
}
