package models;

import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import eye.ss.ServerEnv;
import eye.transfer.EyeData;
import eye.transfer.EyeRecord;
import eye.transfer.EyeType;
import eye.util.UserException;
import eye.util.logging.Log;

public class Macros extends Editors<Strategy> {

	public void addImportsToStrategy(final EyeData strat) {

		ImportLambda lamba = new ImportLambda();
		//Avoid concurrent modification;
		lamba.init(strat);

	}

	@Override
	public Class getChildClass() {
		return Strategy.class;
	}

	@Override
	public EyeType getEyeType() {
		return EyeType.macro;
	}

	public void replaceKnownRefs(EyeRecord record) {

		for (EyeData strat : record.findAll("strategy", EyeData.class)) {
			//All refs must be top level, and are injected into each strategy
			//(can't remember right now why we could have two strategies, but trusting the code.
			for (String key : record.contents.keySet()) {
				if (key.startsWith("ref-")) {
					Object cand = record.get(key);
					if (cand instanceof Number) {
						Number val = record.getLong(key);
						if (val != null) {
							Strategy refContent = find(val.longValue());
							if (refContent == null) {
								throw new UserException(key.substring(4) + " has disapeared, please pick a new value",
										false);
							}

							EyeData d = ServerEnv.loadRecord(refContent.content);
							strat.set(key, d);
						}
					}
				}
			}
		}

	}

	public final class ImportLambda {

		private final HashMap<String, EyeData> macros = new HashMap();
		EyeData target;

		public void importFormula(String value, String idText) {
			Long id = Long.parseLong(idText);
			if (macros.get(value) == null) {
				Strategy m = find(id);
				EyeData d = ServerEnv.loadRecord(m.content);

				if (d.getValues().size() > 1) {
					throw new UserException("Cannot use " + m.label + " as a macro, becuase it has multiple tabs",
							true);
				}
				EyeData child = (EyeData) d.getFirstValue();
				apply(child);
				child.setName(value);
				macros.put(child.getName(), child);
			}
		}

		public void init(EyeData target) {
			this.target = target;
			apply(target);
			//Avoid concurrent by adding later.
			for (Entry<String, EyeData> entry : macros.entrySet()) {
				target.set(entry.getKey(), entry.getValue());
			}
		}

		protected void apply(String value) {

			if (value.startsWith("Import_")) {
				String idText = value.substring("Import_".length());
				importFormula(value, idText);
			} else if (value.endsWith("_Macro")) {
				String id = value.substring(value.indexOf("-") + 1, value.indexOf("_Macro"));
				importFormula(value, id);
			} else {
				if (value.contains("Import_") || value.contains("_Macro")) {
					Log.warning(
							"Failed to detect embedded lambda " + value + "\n\n Full Context is \n" + target.toYaml());

					throw new IllegalStateException(
							"We failed to correctly dereference an embedded formula. Please contact support, as this should not happen");
				}
			}
		}

		private void apply(EyeData arg) {
			for (Entry<String, Object> entry : arg.getEntries()) {
				Object val = entry.getValue();
				if (val instanceof String) {
					apply((String) val);
				} else if (val instanceof List) {
					apply((List) val);
				} else if (val instanceof EyeData) {
					apply((EyeData) val);
				}
			}
		}

		private void apply(List list) {
			for (int i = 0; i < list.size(); i++) {
				Object val = list.get(i);
				if (val instanceof String) {
					apply((String) val);
				} else if (list.get(i) instanceof EyeData) {
					apply((EyeData) val);
				} else if (val instanceof List) {
					apply((List) val);
				}
			}
		}

	}
}
