package models;

import static eye.EyeConstants.*;

import java.util.List;

import eye.play.control.PlayControl;
import eye.ss.ServerEnv;
import eye.transfer.EyeData;
import eye.transfer.EyeRecord;
import eye.transfer.EyeType;
import eye.util.UserException;
import eye.util.collection.ListUtil;

public class PickFilters<G extends PickFilter> extends Editors<G> {

	public void addReferences(EyeRecord input) {
		Long tradingModelId = input.getLong("tradingModel");
		if (tradingModelId != null) {
			Strack strack = PlayControl.tradingModels.get(tradingModelId);
			if (strack == null) {
				throw new UserException(
						"While loading " + input.getLabel() + " bad trading model reference:" + tradingModelId, false);
			}

			EyeData strackContent = ServerEnv.loadRecord(strack.content);
			input.set("tradingModel", strackContent);
		}
		addFakeTickers(input);
		addTearSheet(input);

	}

	public void addTearSheet(EyeRecord r) {
		Number tearSheet = r.get("tearSheet");
		if (tearSheet != null) {
			Strategy m = PlayControl.macros.find(tearSheet.longValue());
			EyeData content = r.get("strategy");

			EyeData data = PlayControl.getEnv().yaml.loadData(m.content);
			data.setName("stockTemplate");
			content.add(data);
		}
	}

	public void bindTradingRules(G filter, EyeRecord r) {
		//Get arround the dropping of empty refs.
		if (r.updateModel) {
			filter.tradingModel = null;
			filter.stopGain = null;
			filter.stopLoss = null;
			filter.rebalance = null;
			filter.maxHoldings = 0;

		}

		r.maybeBind(filter, "maxHoldings", "max-holdings");
		r.maybeBind(filter, "tradingModel");
		r.maybeBind(filter, "stopGain", REF_GAIN);
		r.maybeBind(filter, "stopLoss", REF_STOP);
		r.maybeBind(filter, "rebalance", REF_REBALANCE);
		r.bind(filter, "fakeTickers", "fake-tickers");
		r.bind(filter, "tradingCost", "trading-cost");
	}

	public List<G> getAllOwned(PickFilter owner) {
		List<G> accum = createQuery().findByOwner("master", owner);

		return accum;
	}

	@Override
	public EyeType getEyeType() {
		return EyeType.pickFilter;
	}

	public List<G> getMyOwned(Long ownerId) {
		List<G> accum = createQuery().findByOwnerAndClause("account", PlayControl.getEnv().account,
				ownerId + " = target.master.id");

		return accum;
	}

	@Override
	protected G bind(G filter, EyeRecord r) {
		assert filter != null;
		filter = super.bind(filter, r);
		assert filter != null;
		r.bind(filter, "greatScore");
		r.bind(filter, "badScore");
		r.maybeBind(filter, SORT_BY);
		r.maybeBind(filter, SORT_BY_MAX);
		r.maybeBind(filter, SORT_BY_LIST);
		r.maybeBind(filter, CUSTOM_SIZE);
		bindTradingRules(filter, r);
		bindOwnership(filter, r);
		bindMeta(filter, r);
		return filter;
	}

	private void addFakeTickers(EyeRecord r) {
		List<Number> l = r.remove("fake-tickers");
		if (ListUtil.isEmpty(l)) {
			return;
		}

		int index = 1;
		EyeData fakeTickers = new EyeData();
		fakeTickers.setName("fake-tickers");
		for (Object id : l) {
			EyeRecord fake;
			if (id instanceof PickFilter) {
				fake = PlayControl.pickFilters.load((PickFilter) id);
			} else {
				fake = PlayControl.pickFilters.load(((Number) id).longValue());
			}
			String name = "B" + index;
			++index;
			fake.setName(name);
			fakeTickers.add(fake);
		}
		r.add(fakeTickers);
	}

	private void bindOwnership(G filter, EyeRecord r) {

		if (r.updateModel) {
			Long owner = r.getLong("master-homework");
			if (owner != null) {
				filter.master = PlayControl.pickFilters.get(owner);
				if (filter.master == null) {
					throw new IllegalStateException("Homework " + owner + " seems to no longer exist");
				}
				return;
			}
			owner = r.getLong("master-course");
			if (owner != null) {
				filter.master = PlayControl.courses.get(owner);
			}

		} else {
			if (filter.master != null) {
				if (filter.master instanceof Course) {
					r.set("master-course", filter.master.id);
				} else {
					r.set("master-homework", filter.master.id);
				}
			}
		}
	}
}
