package models;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.apache.commons.io.FileUtils;

import com.objectdb.Doctor;

import eye.EyeConstants;
import eye.play.control.PlayControl;
import eye.service.ServiceEnv;
import eye.ss.ServerEnv;
import eye.ss.database.DatabaseService;
import eye.util.DateUtil;
import eye.util.ExceptionUtil;
import eye.util.logging.Log;
import eye.util.logging.Log.Cat;

public class PlayDbService extends DatabaseService {

	public static String DATABASE_PATH = "./conf/db/play-prod/";

	public static EntityManagerFactory playEmf;

	public static PlayDbService get() {
		return ServerEnv.get().requireService(PlayDbService.class);
	}

	@Override
	public synchronized void backup() {
		backup(DATABASE_PATH);

	}

	@Override
	public void commit() {
		EntityTransaction t = getTransaction();
		if (t.isActive()) {
			if (ServiceEnv.get() instanceof PlayControl) {
				Account account = PlayControl.getEnv().account;
				if (account != null && DateUtil.isToday(account.lastUpdate) == false) {
					account.lastUpdate = new Date();
				}
			}
			t.commit();
			Log.config("Commit on " + playEmf.hashCode(), Cat.DB);
		} else {
			Log.warning(new Exception("DB commit has no effect"), Log.Cat.DB);
		}

	}

	public void copy() {

		Log.config("Copy Database", Cat.DB);
		playEmf.close();
		File prodDbBackup = new File(DATABASE_PATH + "backup.db.odb");
		File prodDb = new File("./conf/db/play-prod/db.odb");
		try {
			FileUtils.copyFile(prodDb, prodDbBackup);
		} catch (IOException e) {
			throw ExceptionUtil.wrap(e);
		} finally {

			reconnect();
		}
	}

	@Override
	public EntityManagerFactory getEmf() {
		if (playEmf == null) {
			globalInit();
		}
		return playEmf;
	}

	@Override
	public void reconnect() {

		Log.config("reconnect", Cat.DB);
		try {
			// /////////////////////////////////////
			// Global Portfolio
			// ////////////////////////////////////

			if (playEmf != null) {
				playEmf.close();
			}
			File prod = new File(DATABASE_PATH);

			if (prod.isDirectory()) {
				File prodDb = new File(DATABASE_PATH + "db.odb");
				if (!prodDb.exists()) {
					File backup = new File(DATABASE_PATH + "backup.db.odb");
					// overrite backup with repair in order not to modify repair at all.
					File repair = new File(DATABASE_PATH + "repair.odb");
					if (repair.exists()) {
						FileUtils.copyFile(repair, backup);
						backup = new File(backup.getPath());
					}
					if (backup.exists()) {
						Log.warning("Repairing production database. Please delete the repair.odb after you are done",
								Cat.DB);
						String[] args = { repair.getAbsolutePath(), prodDb.getAbsolutePath() };
						Doctor.main(args);
						reconnect();
					} else {
						throw new IllegalStateException("Can no longer create production databases from scratch");
					}
				} else {

					Map<String, Object> properties = new HashMap();
					properties.put("javax.persistence.lock.timeout", 2000);

					PlayDbService.playEmf = Persistence
							.createEntityManagerFactory("objectdb:" + prodDb.getAbsolutePath() + ";", properties);

					PlayControl.global = PlayControl.accounts.require(EyeConstants.GLOBAL);
					PlayControl.test = PlayControl.accounts.require("test");
					assert PlayControl.global != null;
					assert PlayControl.test != null;

				}
			} else {
				throw new IllegalStateException("Can no longer create production databases from scratch");
			}

		} catch (Exception e) {

			throw ExceptionUtil.wrap(e);
		}
	}

	private synchronized void globalInit() {
		if (playEmf == null) {
			reconnect();
		}
	}

}
