package models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import eye.play.control.PlayControl;
import eye.util.LoginEx;
import eye.util.NamedObject;
import eye.util.StringUtil;
import eye.util.UserException;
import eye.util.logging.Log;

public class ShadowLogin extends NamedObject {
	private static Map<String, ShadowLogin> allShadows;
	public String password;
	public String account;

	public static Account checkForShadowAccount(String name, String password) {
		ShadowLogin login = getShadowLoginMap().get(name);
		allShadows = null;
		if (login == null) {
			return null;
		} else {
			if (!StringUtil.equals(login.password, password)) {
				throw new LoginEx("Password " + password + " doesn't match the expected password for " + name);
			}
			Log.info(name + " shadow login for account " + login.account);
			Account account = PlayControl.accounts.get(login.account);
			return account;
		}
	}

	public static void setShadowLogins(Account account, String... loginArray) {

		ArrayList<String> logins = new ArrayList();

		Map<String, ShadowLogin> map = getShadowLoginMap();
		for (int i = 0; i < loginArray.length; i++) {
			String userName = loginArray[i].trim();
			ShadowLogin matches = map.get(userName);
			if (matches != null && matches.account.equals(account.getName()) == false) {
				throw new UserException("We already have a login with the user name " + userName
						+ " which is associated with another account\n " + matches.account + ".", true);
			}
			logins.add(userName);
			logins.add(loginArray[++i]);
		}
		account.shadowLogins = logins;
		addLogins(account, allShadows);

	}

	private static void addLogins(Account account, Map<String, ShadowLogin> map) {
		if (account.shadowLogins == null) {
			return;
		}

		List<String> source = account.shadowLogins;
		for (int i = 0; i < source.size(); i++) {
			ShadowLogin login = new ShadowLogin();
			login.setName(source.get(i++));
			login.password = source.get(i);
			login.account = account.getName();
			map.put(login.getName(), login);
		}
	}

	private static Map<String, ShadowLogin> getShadowLoginMap() {
		if (allShadows == null) {
			allShadows = new HashMap();
			for (Account account : PlayControl.accounts.findAll()) {
				ShadowLogin.addLogins(account, allShadows);
			}
		}
		return allShadows;
	}

}