package models;

import java.util.Collections;
import java.util.List;

import eye.play.control.PlayControl;
import eye.service.stock.TickerService;
import eye.transfer.EyeData;
import eye.transfer.EyeRecord;
import eye.transfer.EyeTable;
import eye.transfer.EyeType;
import eye.util.StopWatch;

public class StockPicks extends AccountMembers<StockPick> {

	// Watchlist
	public Watchlist watchlist;

	public StockPick createTransientPick(String ticker, Strack tradingModel) {
		StockPick stockPick = new StockPick();
		stockPick.security = ticker;
		stockPick.label = TickerService.get().getTickerById(ticker).getLabel();
		stockPick.account = PlayControl.getEnv().account;
		return stockPick;
	}

	@Override
	public EyeType getEyeType() {
		return EyeType.stockPick;
	}

	@Override
	public EyeRecord getModels() {
		StopWatch watch = new StopWatch("available records");
		EyeRecord r = new EyeRecord();
		r.setName("availableRecords");
		EyeTable table = createModelTable();
		r.add(table);
		watch.report("Created strack file");
		List<StockPick> strats = findModels();
		Collections.sort(strats);

		watch.report("got list");
		for (StockPick strat : strats) {
			// We only want strategies associated with tactics here.
			if (isAvailable(strat)) {
				addToTable(table, strat, strat.watchlist.getLabel(), getEyeType());
			}
		}

		watch.report("done");

		return r;
	}

	public StockPick getStockPick(String watchlistName, String ticker) {
		String name = watchlistName + "-" + ticker;
		StockPick strack = createQuery().findByName(name);
		return strack;
	}

	@Override
	protected StockPick bind(StockPick strat, EyeRecord r) {
		strat = super.bind(strat, r);
		if (r.updateModel) {
			String ownerName = r.get(EyeData.WATCHLIST_NAME);
			Number ownerId = r.get(EyeData.WATCHLIST_ID);
			Watchlist list = null;
			if (ownerId != null) {
				list = PlayControl.watchlists.find(ownerId.longValue());
			} else if (ownerName != null) {
				list = PlayControl.watchlists.require(ownerName);
			}
			assert list != null;
			strat.watchlist = list;
			Boolean active = r.get("active");
			if (active == Boolean.FALSE) {
				strat.becomeOut();
			}
		} else {
			super.bind(strat, r);
			if (strat.watchlist != null) {
				r.set("watchlist.id", strat.watchlist.id);
				r.set("watchlist.label", strat.watchlist.getLabel());
				r.set("watchlist.description", strat.watchlist.description);
			}
		}
		return strat;
	}

	@Override
	protected boolean isAvailable(StockPick strat) {
		return strat.watchlist != null;
	}

	@Override
	protected void setAccount(StockPick child, Account account) {
		child.account = account;

	}
}
