package models;

import java.util.Date;
import java.util.HashMap;

import javax.jdo.annotations.Index;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

import eye.play.control.PlayControl;
import eye.util.charactoristic.HasMeta;
import eye.util.logging.Log;

@Entity
// Denormalized for ease of use.
public class Strategy extends EyeModel implements HasMeta {
	@ManyToOne
	@Index
	public Account account;

	@Transient
	transient public boolean copied;

	public String content;

	public Date lastModified;
	@Index
	@ManyToMany
	/**
	 * In most cases the account is the owner, however in specialized situations, such as course
	 * homework, the owner is not actually the account.
	 */
	public EyeModel master;

	private HashMap<String, Object> metadata;

	public Strategy() {
	}

	@Override
	public int compareTo(EyeModel model) {
		Strategy m = (Strategy) model;
		assert m.account != null : m + " needs an account";
		assert account != null : this + " of type " + this.getClass().getSimpleName() + "  needs an account";
		if (m.account == account) {
			return super.compareTo(model);
		} else {
			if (account == null) {
				Log.warning("how did " + this + "get a null account?");
				return 0;
			}
			return account.compareTo(m.account);
		}
	}

	@Override
	public <X extends EyeModel> X copy() {
		Strategy me = super.copy();
		me.account = PlayControl.getEnv().account;
		if (me.getMetadata() != null) {
			me.setMetadata((HashMap<String, Object>) getMetadata().clone());
		}
		return (X) me;
	}

	@Override
	public HashMap<String, Object> getMetadata() {
		return metadata;
	}

	@Override
	public EyeModel getOwner() {
		return account;
	}

	public String getSimpleLabel() {
		String label = getLabel();
		if (label == null) {
			return "unknown";
		}
		int lastColon = label.lastIndexOf(":");
		if (lastColon == -1) {
			return label;
		} else {
			return label.substring(lastColon + 1);
		}
	}

	@Override
	public void moveTo(Account target) {
		account = target;
	}

	public void setAccount(Account account) {
		this.account = account;
	}

	public void setMaster(EyeModel owner) {
		if (owner instanceof Course) {
			Course course = (Course) owner;
			course.addHomework((PickFilter) this);
		}
		assert owner instanceof Course || owner instanceof PickFilter;
		master = owner;
	}

	@Override
	public void setMetadata(HashMap<String, Object> metadata) {
		this.metadata = metadata;
	}

	@Override
	public String toString() {
		String accum = getLabel();
		if (account != null) {
			accum += " " + account.getLabel();
		}
		return accum;
	}

	protected Strategy findByLabel(String label) {
		return (Strategy) createQuery().findByLabel(label, "account", account);

	}
}
