package models;

import java.util.Date;
import java.util.List;

import eye.play.control.PlayControl;
import eye.service.stock.SignalReason;
import eye.service.stock.TickerService;
import eye.service.stock.TickerService.Ticker;
import eye.transfer.EyeData;
import eye.transfer.EyeRecord;
import eye.transfer.EyeTable;
import eye.transfer.EyeType;

public class Watchlists extends PickFilters<Watchlist> {

	/**
	 * Create table of tickers for user display in web.
	 */
	public EyeTable createUserTable(EyeTable tickers) {
		EyeTable userTable = new EyeTable();
		userTable.addColumn("ticker-id", EyeType.String);
		userTable.addColumn("company", EyeType.String);
		userTable.addColumn("action", EyeType.String);
		userTable.addColumn("reason", EyeType.String);
		TickerService tickerSrv = TickerService.get();

		for (int rowIdx = 0; rowIdx < tickers.size(); rowIdx++) {
			String tickerId = (String) tickers.getValueAt(rowIdx, 0);
			Ticker ticker = tickerSrv.getTickerById(tickerId);
			String company = ticker.getHtmlLabel();
			String action = tickers.getValueAt(rowIdx, "action");
			String[] reasons = tickers.getValueAt(rowIdx, "reason");
			String reason = SignalReason.toPrettyString(reasons);
			userTable.addRow(tickerId, company, action, reason);
		}

		userTable.setName("watchlist");
		return userTable;

	}

	public Watchlist ensure(String label) {
		Watchlist list = findByLabel(label);
		if (list != null) {
			return list;
		}
		list = create(null, label, "Imported stocks");
		list.content = "{}";
		return list;
	}

	public Watchlist ensure(String label, String description) {
		Watchlist list = findByLabel(label);
		if (list != null) {
			return list;
		}
		list = create(null, label, description);
		list.content = "{}";
		return list;
	}

	@Override
	public List<Watchlist> findModels() {

		return createQuery().findByOwner("account", PlayControl.getEnv().account);
	}

	@Override
	public Class<Watchlist> getChildClass() {
		return Watchlist.class;
	}

	@Override
	public EyeType getEyeType() {
		return EyeType.watchlist;
	}

	public void updateRecord(Watchlist list, EyeRecord r) {

		EyeData d = new EyeData().setName("inFilters");
		r.add(d);
		list.addTradingRules(r);

		if (list.inFilters != null) {

			for (PickFilter f : list.inFilters) {
				d.set(f.id + "", f.getStrategy());
			}
			if (list.inFilters.size() > 0) {
				list.inFilters.get(0).addTradingRules(r);

			}
		}

		if (list.whitelist != null) {
			r.set("whitelist", list.whitelist);
		}
		if (list.blacklist != null) {
			r.set("blacklist", list.blacklist);
		}

		if (list.outFilters != null) {
			EyeData out = new EyeData().setName("outFilters");
			for (PickFilter f : list.outFilters) {
				out.set(f.id + "", f.getStrategy());
			}
			r.add(out);
		}

		boolean hasAsOf = r.get("as-of-date") != null;
		EyeTable updateTable = list.prepareUpdateTable(!hasAsOf);
		if (hasAsOf == false) {
			r.set("as-of-date", eye.util.DateUtil.serverFormat.format(new Date()));
		}

		r.add(updateTable);

	}

	@Override
	protected Watchlist bind(Watchlist watchlist, EyeRecord r) {
		r.bind(watchlist, "inFilters");
		if (watchlist.inFilters.size() > 0) {
			PickFilter primary = watchlist.inFilters.get(0);
			watchlist.setPrimary(primary);

		}
		watchlist = super.bind(watchlist, r);

		r.bind(watchlist, "outFilters");

		if (r.updateModel) {
			watchlist.changeWhitelist((List<String>) r.require("whitelist"));
		} else {
			r.bind(watchlist, "whitelist");
		}
		r.bind(watchlist, "blacklist");

		return watchlist;
	}

	@Override
	protected EyeRecord prepareForUpdate(Long id) {
		Watchlist list = find(id);
		EyeRecord update = load(list);
		updateRecord(list, update);
		return update;
	}

}
